/* seealso service description */
body, h1, h2, th, td { 
    font-family: sans-serif;
    color: #333;
}
a { text-decoration: none; color: #08C; }
a:hover { text-decoration: underline; }
a:visited { color: #08C; }

h1, h2, h1 a, h1 a:visited { color: #96c458; }
h1 { font-size: 1.5em; }
h2 { font-size: 1.2em; }

table, .code, p { margin: 0em 0.5em 0em; }
table { 
    border-collapse:collapse;
    margin-bottom: 0.5em; 
}
td, th { padding: 4px; }
th { text-align: right; vertical-align: top; }

p { padding-bottom: 0.5em; padding-left: 0.5em; }

pre, .code {
    background: #ddd; 
    padding: 4px;
    margin: 0;
}
#display-styles {
    margin-top: 0.5em;
}
#display {
    background: #fff;
    border: 1px dotted #ddd;
    padding: 4px;
}
.footer {
    font-size: small;
    color: #666;
    margin-top: 1em;
    padding: 0.5em;
}

/* xmlverbatim.css */
.xmlverb-default          { color: #333333; background-color: #ffffff;
                            font-family: monospace }
.xmlverb-element-name     { color: #990000 }
.xmlverb-element-nsprefix { color: #666600 }
.xmlverb-attr-name        { color: #660000 }
.xmlverb-attr-content     { color: #000099; font-weight: bold }
.xmlverb-ns-name          { color: #666600 }
.xmlverb-ns-uri           { color: #330099 }
.xmlverb-text             { color: #000000; font-weight: bold }
.xmlverb-comment          { color: #006600; font-style: italic }
.xmlverb-pi-name          { color: #006600; font-style: italic }
.xmlverb-pi-content       { color: #006666; font-style: italic }
